// Hilbert.cpp: implementation of the Hilbert class.
//
//////////////////////////////////////////////////////////////////////

//#include "Hilbert.h"

//-------------

// Hilbert.h: interface for the Hilbert class.
//
//////////////////////////////////////////////////////////////////////

#define BYTE unsigned char
class Hilbert  
{
public:
  int jh;
  int ih;
  int nh;
  int th;
  int * bh;
  int * ah;
  Hilbert();
  virtual ~Hilbert();

  void hilbertz(int * src, int * dst, int height, int width, int rnd);

  void hilbertn(int * src, int* dst, int height, int width, int rnd);

  void OnHilbertZ(int* src, int* dst, int height, int width);

  void OnHilbertN(int* src, int* dst, int height, int width);

  void h1(int step, int width, int height);

  void h2(int step, int width, int height);

  void h3(int step, int width, int height);

  void h4(int step, int width, int height);

  void fh1(int step, int width, int height);

  void fh2(int step, int width, int height);

  void fh3(int step, int width, int height);

  void fh4(int step, int width, int height);
};

//-------------















//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Hilbert::Hilbert()
{
  jh=0;
  ih=0;
  nh=0;
  th=0;
  ah = new int[1024 * 1024];//????BUFFER 
  bh = new int[1024 * 1024];
}

Hilbert::~Hilbert()
{

}

//hilbert???????
Hilbert::hilbertz(int* src, int* dst, int height, int width, int rnd)
{
  int i;
  int* temp = new int[height*width];
  for (i = 0; i < height * width; i++) temp[i] = src[i];
  int* temp2 = new int[height*width];
  for (; rnd > 0; rnd--)
  {
    OnHilbertZ(temp, temp2, height, width);
    for (i = 0; i < height * width; i++) temp[i] = temp2[i];
  }
  for (i = 0; i < height * width; i++) dst[i] = temp[i];
  delete temp;
  delete temp2;
}
//hilbert???????
Hilbert::hilbertn(int* src, int* dst, int height, int width, int rnd)
{
  int i;
  int* temp = new int[height*width];
  for (i = 0; i < height * width; i++) temp[i] = src[i];
  int* temp2 = new int[height*width];
  for (; rnd > 0; rnd--)
  {
    OnHilbertN(temp, temp2, height, width);
    for (i = 0; i < height * width; i++) temp[i] = temp2[i];
  }
  for (i = 0; i < height * width; i++) dst[i] = temp[i];
  delete temp;
  delete temp2;
}
//hilbert?????
Hilbert::OnHilbertZ(int* src, int* dst, int height, int width)
{
  // TODO: Add your command handler code here

  th = 0;
  nh = width;
  for (ih = 0; ih < height; ih++)
    for (jh = 0; jh < width; jh++)
    {
      ah[(ih * width + jh)] = src[(ih * width + jh)];
    }//??????
  ih = nh - 1; jh = 0;//????????
  h2(nh, width, height);
  for (ih = 0; ih < height; ih++)
    for (jh = 0; jh < width; jh++)
    {
      dst[(ih * width + jh)] = bh[(ih * width + jh)];
    }//??HILBERT???
  th = 0;//????????
}
//hilbert?????
Hilbert::OnHilbertN(int* src, int* dst, int height, int width)
{
  // TODO: Add your command handler code here

  th = 0;
  nh = width;
  for (ih = 0; ih < height; ih++)
    for (jh = 0; jh < width; jh++)
    {
      ah[(ih * width + jh)] = src[(ih * width + jh)];
    }//??????
  ih = nh - 1; jh = 0;//????????
  fh2(nh, width, height);
  for (ih = 0; ih < height; ih++)
    for (jh = 0; jh < width; jh++)
    {
      dst[(ih * width + jh)] = bh[(ih * width + jh)];
    }//??HILBERT???
  th = 0;//????????
}
//HILBERT??????
Hilbert::h1(int step, int width, int height)//HILBERT??????
{
  if (step == 2)
  {
    bh[((th / nh) * width + ((th++) % nh))] = ah[(ih * width + jh)];
    bh[((th / nh) * width + ((th++) % nh))] = ah[(ih * width + (++jh))];
    bh[((th / nh) * width + ((th++) % nh))] = ah[((--ih) * width + jh)];
    bh[((th / nh) * width + ((th++) % nh))] = ah[(ih * width + (--jh))];
  }
  else
  {
    h2(step / 2, width, height);
    jh++; h1(step / 2, width, height);
    ih--; h1(step / 2, width, height);
    jh--; h3(step / 2, width, height);
  }
}
//HILBERT??????
Hilbert::h2(int step, int width, int height)//HILBERT??????
{
  if (step == 2)
  {
    bh[((th / nh) * width + ((th++) % nh))] = ah[(ih * width + jh)];
    bh[((th / nh) * width + ((th++) % nh))] = ah[((--ih) * width + jh)];
    bh[((th / nh) * width + ((th++) % nh))] = ah[(ih * width + (++jh))];
    bh[((th / nh) * width + ((th++) % nh))] = ah[((++ih) * width + jh)];
  }
  else
  {
    h1(step / 2, width, height);
    ih--; h2(step / 2, width, height);
    jh++; h2(step / 2, width, height);
    ih++; h4(step / 2, width, height);
  }
}
//HILBERT??????
Hilbert::h3(int step, int width, int height)//HILBERT??????
{
  if (step == 2)
  {
    bh[((th / nh) * width + ((th++) % nh))] = ah[(ih * width + jh)];
    bh[((th / nh) * width + ((th++) % nh))] = ah[((++ih) * width + jh)];
    bh[((th / nh) * width + ((th++) % nh))] = ah[(ih * width + (--jh))];
    bh[((th / nh) * width + ((th++) % nh))] = ah[((--ih) * width + jh)];
  }
  else
  {
    h4(step / 2, width, height);
    ih++; h3(step / 2, width, height);
    jh--; h3(step / 2, width, height);
    ih--; h1(step / 2, width, height);
  }
}
//HILBERT??????
Hilbert::h4(int step, int width, int height)//HILBERT??????
{
  if (step == 2)
  {
    bh[((th / nh) * width + ((th++) % nh))] = ah[(ih * width + jh)];
    bh[((th / nh) * width + ((th++) % nh))] = ah[(ih * width + (--jh))];
    bh[((th / nh) * width + ((th++) % nh))] = ah[((++ih) * width + jh)];
    bh[((th / nh) * width + ((th++) % nh))] = ah[(ih * width + (++jh))];
  }
  else
  {
    h3(step / 2, width, height);
    jh--; h4(step / 2, width, height);
    ih++; h4(step / 2, width, height);
    jh++; h2(step / 2, width, height);
  }
}
//HILBERT??????
Hilbert::fh1(int step, int width, int height)//HILBERT??????
{
  if (step == 2)
  {
    bh[(ih * width + jh)] = ah[((th / nh) * width + ((th++) % nh))];
    bh[(ih * width + (++jh))] = ah[((th / nh) * width + ((th++) % nh))];
    bh[((--ih) * width + jh)] = ah[((th / nh) * width + ((th++) % nh))];
    bh[(ih * width + (--jh))] = ah[((th / nh) * width + ((th++) % nh))];
  }
  else
  {
    fh2(step / 2, width, height);
    jh++; fh1(step / 2, width, height);
    ih--; fh1(step / 2, width, height);
    jh--; fh3(step / 2, width, height);
  }
}
//HILBERT??????
Hilbert::fh2(int step, int width, int height)//HILBERT??????
{
  if (step == 2)
  {
    bh[(ih * width + jh)] = ah[((th / nh) * width + ((th++) % nh))];
    bh[((--ih) * width + jh)] = ah[((th / nh) * width + ((th++) % nh))];
    bh[(ih * width + (++jh))] = ah[((th / nh) * width + ((th++) % nh))];
    bh[((++ih) * width + jh)] = ah[((th / nh) * width + ((th++) % nh))];
  }
  else
  {
    fh1(step / 2, width, height);
    ih--; fh2(step / 2, width, height);
    jh++; fh2(step / 2, width, height);
    ih++; fh4(step / 2, width, height);
  }
}
//HILBERT??????
Hilbert::fh3(int step, int width, int height)//HILBERT??????
{
  if (step == 2)
  {
    bh[(ih * width + jh)] = ah[((th / nh) * width + ((th++) % nh))];
    bh[((++ih) * width + jh)] = ah[((th / nh) * width + ((th++) % nh))];
    bh[(ih * width + (--jh))] = ah[((th / nh) * width + ((th++) % nh))];
    bh[((--ih) * width + jh)] = ah[((th / nh) * width + ((th++) % nh))];
  }
  else
  {
    fh4(step / 2, width, height);
    ih++; fh3(step / 2, width, height);
    jh--; fh3(step / 2, width, height);
    ih--; fh1(step / 2, width, height);
  }
}
//HILBERT??????
Hilbert::fh4(int step, int width, int height)//HILBERT??????
{
  if (step == 2)
  {
    bh[(ih * width + jh)] = ah[((th / nh) * width + ((th++) % nh))];
    bh[(ih * width + (--jh))] = ah[((th / nh) * width + ((th++) % nh))];
    bh[((++ih) * width + jh)] = ah[((th / nh) * width + ((th++) % nh))];
    bh[(ih * width + (++jh))] = ah[((th / nh) * width + ((th++) % nh))];
  }
  else
  {
    fh3(step / 2, width, height);
    jh--; fh4(step / 2, width, height);
    ih++; fh4(step / 2, width, height);
    jh++; fh2(step / 2, width, height);
  }
}